@@ -3,7 +3,7 @@
 #include <stdbool.h>
 #include <string.h>
 #include <ctype.h>
-#include "llhttp/llhttp.h"
+#include "lib/llhttp.h"
 
 #define STRING(x) #x
 #define XSTRING(x) STRING(x)
@@ -116,35 +116,25 @@ llhttp_settings_t parser_settings = {
     .on_chunk_complete = parser_on_chunk_complete,
 };
 
-static PyObject *
-request_reset(PyObject *self) {
-    llhttp_t *llhttp = &((parser_object*)self)->llhttp;
-    llhttp_init(llhttp, HTTP_REQUEST, &parser_settings);
-    llhttp->data = self;
-    Py_RETURN_NONE;
-}
-
 static PyObject *
 request_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
     PyObject *self = type->tp_alloc(type, 0);
-    if (self)
-        request_reset(self);
+    if (self) {
+        llhttp_t *llhttp = &((parser_object*)self)->llhttp;
+        llhttp_init(llhttp, HTTP_REQUEST, &parser_settings);
+        llhttp->data = self;
+    }
     return self;
 }
 
-static PyObject *
-response_reset(PyObject *self) {
-    llhttp_t *llhttp = &((parser_object*)self)->llhttp;
-    llhttp_init(llhttp, HTTP_RESPONSE, &parser_settings);
-    llhttp->data = self;
-    Py_RETURN_NONE;
-}
-
 static PyObject *
 response_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
     PyObject *self = type->tp_alloc(type, 0);
-    if (self)
-        response_reset(self);
+    if (self) {
+        llhttp_t *llhttp = &((parser_object*)self)->llhttp;
+        llhttp_init(llhttp, HTTP_RESPONSE, &parser_settings);
+        llhttp->data = self;
+    }
     return self;
 }
 
@@ -215,7 +205,15 @@ parser_finish(PyObject *self) {
     return NULL;
 }
 
-static PyObject * parser_reset(PyObject *self);
+static PyObject *
+parser_reset(PyObject *self) {
+    llhttp_t *llhttp = &((parser_object*)self)->llhttp;
+    uint8_t lenient_flags = llhttp->lenient_flags;
+    llhttp_init(llhttp, llhttp->type, &parser_settings);
+    llhttp->lenient_flags = lenient_flags;
+    llhttp->data = self;
+    Py_RETURN_NONE;
+}
 
 static PyObject * parser_dummy_noargs(PyObject *self) { Py_RETURN_NONE; }
 static PyObject * parser_dummy_onearg(PyObject *self, PyObject *arg) { Py_RETURN_NONE; }
@@ -367,55 +365,45 @@ parser_dealloc(PyObject *self) {
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
-static PyTypeObject request_type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    .tp_name = "llhttp.Request",
-    .tp_doc = "llhttp request parser",
-    .tp_basicsize = sizeof(parser_object),
-    .tp_getattro = PyObject_GenericGetAttr,
-    .tp_setattro = PyObject_GenericSetAttr,
-    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
-    .tp_new = request_new,
-    .tp_dealloc = parser_dealloc,
-    .tp_methods = parser_methods,
-    .tp_getset = parser_getset,
+static PyType_Slot request_slots[] = {
+    {Py_tp_doc, "llhttp request parser"},
+    {Py_tp_new, request_new},
+    {Py_tp_dealloc, parser_dealloc},
+    {Py_tp_methods, parser_methods},
+    {Py_tp_getset, parser_getset},
+    {0, 0},
 };
 
-static PyTypeObject response_type = {
-    PyVarObject_HEAD_INIT(NULL, 0)
-    .tp_name = "llhttp.Response",
-    .tp_doc = "llhttp response parser",
-    .tp_basicsize = sizeof(parser_object),
-    .tp_getattro = PyObject_GenericGetAttr,
-    .tp_setattro = PyObject_GenericSetAttr,
-    .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
-    .tp_new = response_new,
-    .tp_dealloc = parser_dealloc,
-    .tp_methods = parser_methods,
-    .tp_getset = parser_getset,
+static PyType_Spec request_spec = {
+    "llhttp.Request",
+    sizeof(parser_object),
+    0,
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+    request_slots,
 };
 
-static PyObject *
-parser_reset(PyObject *self) {
-    if (PyObject_TypeCheck(self, &request_type)) {
-        request_reset(self);
-        Py_RETURN_NONE;
-    }
+static PyType_Slot response_slots[] = {
+    {Py_tp_doc, "llhttp response parser"},
+    {Py_tp_new, response_new},
+    {Py_tp_dealloc, parser_dealloc},
+    {Py_tp_methods, parser_methods},
+    {Py_tp_getset, parser_getset},
+    {0, 0},
+};
 
-    if (PyObject_TypeCheck(self, &response_type)) {
-        response_reset(self);
-        Py_RETURN_NONE;
-    }
-
-    PyErr_SetString(PyExc_TypeError, "not llhttp.Request or llhttp.Response");
-    return NULL;
-}
+static PyType_Spec response_spec = {
+    "llhttp.Response",
+    sizeof(parser_object),
+    0,
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+    response_slots,
+};
 
 static struct PyModuleDef llhttp_module = {
     PyModuleDef_HEAD_INIT,
     .m_name = "llhttp",
     .m_doc = "llhttp wrapper",
-    .m_size = -1
+    .m_size = -1,
 };
 
 static char *
@@ -435,7 +423,7 @@ snake_to_camel(char * string) {
 }
 
 PyMODINIT_FUNC
-PyInit_llhttp(void) {
+PyInit___llhttp(void) {
     PyObject *m = PyModule_Create(&llhttp_module);
     if (!m)
         return NULL;
@@ -468,21 +456,21 @@ HTTP_ERRNO_MAP(HTTP_ERRNO_GEN)
 HTTP_METHOD_MAP(HTTP_METHOD_GEN)
 #undef HTTP_METHOD_GEN
 
-    if (PyType_Ready(&request_type))
+    PyObject *request_type = PyType_FromSpec(&request_spec);
+    if (!request_type)
         goto fail;
 
-    Py_INCREF(&request_type);
-    if (PyModule_AddObject(m, request_type.tp_name + strlen("llhttp."), (PyObject *)&request_type)) {
-        Py_DECREF(&request_type);
+    if (PyModule_AddObject(m, request_spec.name + strlen("llhttp."), request_type)) {
+        Py_DECREF(request_type);
         goto fail;
     }
 
-    if (PyType_Ready(&response_type))
+    PyObject *response_type = PyType_FromSpec(&response_spec);
+    if (!response_type)
         goto fail;
 
-    Py_INCREF(&response_type);
-    if (PyModule_AddObject(m, response_type.tp_name + strlen("llhttp."), (PyObject *)&response_type)) {
-        Py_DECREF(&response_type);
+    if (PyModule_AddObject(m, response_spec.name + strlen("llhttp."), response_type)) {
+        Py_DECREF(response_type);
         goto fail;
     }
 
